@@ -1,19 +1,18 @@
-// Copyright 2018, The Gtk-rs Project Developers.
-// See the COPYRIGHT file at the top-level directory of this distribution.
-// Licensed under the MIT license, see the LICENSE file or <http://opensource.org/licenses/MIT>
+// Take a look at the license at the top of the repository in the LICENSE file.
 
-use futures::channel::{mpsc, oneshot};
-use futures::prelude::*;
-use futures::task;
-use futures::task::Poll;
-use std::marker::Unpin;
-use std::pin;
+use std::{pin, pin::Pin, time::Duration};
 
-use Continue;
-use MainContext;
-use Priority;
-use Source;
+use futures_channel::{mpsc, oneshot};
+use futures_core::{
+    future::{FusedFuture, Future},
+    stream::{FusedStream, Stream},
+    task,
+    task::Poll,
+};
 
+use crate::{ControlFlow, MainContext, Priority, Source};
+
+// rustdoc-stripper-ignore-next
 /// Represents a `Future` around a `glib::Source`. The future will
 /// be resolved once the source has provided a value
 pub struct SourceFuture<F, T> {
@@ -23,8 +22,9 @@ pub struct SourceFuture<F, T> {
 
 impl<F, T: 'static> SourceFuture<F, T>
 where
-    F: FnOnce(oneshot::Sender<T>) -> Source + Send + 'static,
+    F: FnOnce(oneshot::Sender<T>) -> Source + 'static,
 {
+    // rustdoc-stripper-ignore-next
     /// Create a new `SourceFuture`
     ///
     /// The provided closure should return a newly created `glib::Source` when called
@@ -42,7 +42,7 @@ impl<F, T> Unpin for SourceFuture<F, T> {}
 
 impl<F, T> Future for SourceFuture<F, T>
 where
-    F: FnOnce(oneshot::Sender<T>) -> Source + Send + 'static,
+    F: FnOnce(oneshot::Sender<T>) -> Source + 'static,
 {
     type Output = T;
 
@@ -62,7 +62,7 @@ where
 
             // Channel for sending back the Source result to our future here.
             //
-            // In theory we could directly continue polling the
+            // In theory, we could directly continue polling the
             // corresponding task from the Source callback,
             // however this would break at the very least
             // the g_main_current_source() API.
@@ -77,7 +77,7 @@ where
         // At this point we must have a receiver
         let res = {
             let &mut (_, ref mut receiver) = source.as_mut().unwrap();
-            receiver.poll_unpin(ctx)
+            Pin::new(receiver).poll(ctx)
         };
         #[allow(clippy::match_wild_err_arm)]
         match res {
@@ -92,6 +92,19 @@ where
     }
 }
 
+impl<F, T> FusedFuture for SourceFuture<F, T>
+where
+    F: FnOnce(oneshot::Sender<T>) -> Source + 'static,
+{
+    fn is_terminated(&self) -> bool {
+        self.create_source.is_none()
+            && self
+                .source
+                .as_ref()
+                .map_or(true, |(_, receiver)| receiver.is_terminated())
+    }
+}
+
 impl<T, F> Drop for SourceFuture<T, F> {
     fn drop(&mut self) {
         // Get rid of the source, we don't care anymore if it still triggers
@@ -101,65 +114,69 @@ impl<T, F> Drop for SourceFuture<T, F> {
     }
 }
 
+// rustdoc-stripper-ignore-next
 /// Create a `Future` that will resolve after the given number of milliseconds.
 ///
 /// The `Future` must be spawned on an `Executor` backed by a `glib::MainContext`.
-pub fn timeout_future(value: u32) -> Box<dyn Future<Output = ()> + std::marker::Unpin + Send> {
-    timeout_future_with_priority(::PRIORITY_DEFAULT, value)
+pub fn timeout_future(value: Duration) -> Pin<Box<dyn Future<Output = ()> + Send + 'static>> {
+    timeout_future_with_priority(crate::Priority::default(), value)
 }
 
+// rustdoc-stripper-ignore-next
 /// Create a `Future` that will resolve after the given number of milliseconds.
 ///
 /// The `Future` must be spawned on an `Executor` backed by a `glib::MainContext`.
 pub fn timeout_future_with_priority(
     priority: Priority,
-    value: u32,
-) -> Box<dyn Future<Output = ()> + std::marker::Unpin + Send> {
-    Box::new(SourceFuture::new(move |send| {
+    value: Duration,
+) -> Pin<Box<dyn Future<Output = ()> + Send + 'static>> {
+    Box::pin(SourceFuture::new(move |send| {
         let mut send = Some(send);
-        ::timeout_source_new(value, None, priority, move || {
+        crate::timeout_source_new(value, None, priority, move || {
             let _ = send.take().unwrap().send(());
-            Continue(false)
+            ControlFlow::Break
         })
     }))
 }
 
+// rustdoc-stripper-ignore-next
 /// Create a `Future` that will resolve after the given number of seconds.
 ///
 /// The `Future` must be spawned on an `Executor` backed by a `glib::MainContext`.
-pub fn timeout_future_seconds(
-    value: u32,
-) -> Box<dyn Future<Output = ()> + std::marker::Unpin + Send> {
-    timeout_future_seconds_with_priority(::PRIORITY_DEFAULT, value)
+pub fn timeout_future_seconds(value: u32) -> Pin<Box<dyn Future<Output = ()> + Send + 'static>> {
+    timeout_future_seconds_with_priority(crate::Priority::default(), value)
 }
 
+// rustdoc-stripper-ignore-next
 /// Create a `Future` that will resolve after the given number of seconds.
 ///
 /// The `Future` must be spawned on an `Executor` backed by a `glib::MainContext`.
 pub fn timeout_future_seconds_with_priority(
     priority: Priority,
     value: u32,
-) -> Box<dyn Future<Output = ()> + std::marker::Unpin + Send> {
-    Box::new(SourceFuture::new(move |send| {
+) -> Pin<Box<dyn Future<Output = ()> + Send + 'static>> {
+    Box::pin(SourceFuture::new(move |send| {
         let mut send = Some(send);
-        ::timeout_source_new_seconds(value, None, priority, move || {
+        crate::timeout_source_new_seconds(value, None, priority, move || {
             let _ = send.take().unwrap().send(());
-            Continue(false)
+            ControlFlow::Break
         })
     }))
 }
 
+// rustdoc-stripper-ignore-next
 /// Create a `Future` that will resolve once the child process with the given pid exits
 ///
 /// The `Future` will resolve to the pid of the child process and the exit code.
 ///
 /// The `Future` must be spawned on an `Executor` backed by a `glib::MainContext`.
 pub fn child_watch_future(
-    pid: ::Pid,
-) -> Box<dyn Future<Output = (::Pid, i32)> + std::marker::Unpin + Send> {
-    child_watch_future_with_priority(::PRIORITY_DEFAULT, pid)
+    pid: crate::Pid,
+) -> Pin<Box<dyn Future<Output = (crate::Pid, i32)> + Send + 'static>> {
+    child_watch_future_with_priority(crate::Priority::default(), pid)
 }
 
+// rustdoc-stripper-ignore-next
 /// Create a `Future` that will resolve once the child process with the given pid exits
 ///
 /// The `Future` will resolve to the pid of the child process and the exit code.
@@ -167,41 +184,46 @@ pub fn child_watch_future(
 /// The `Future` must be spawned on an `Executor` backed by a `glib::MainContext`.
 pub fn child_watch_future_with_priority(
     priority: Priority,
-    pid: ::Pid,
-) -> Box<dyn Future<Output = (::Pid, i32)> + std::marker::Unpin + Send> {
-    Box::new(SourceFuture::new(move |send| {
+    pid: crate::Pid,
+) -> Pin<Box<dyn Future<Output = (crate::Pid, i32)> + Send + 'static>> {
+    Box::pin(SourceFuture::new(move |send| {
         let mut send = Some(send);
-        ::child_watch_source_new(pid, None, priority, move |pid, code| {
+        crate::child_watch_source_new(pid, None, priority, move |pid, code| {
             let _ = send.take().unwrap().send((pid, code));
         })
     }))
 }
 
-#[cfg(any(unix, feature = "dox"))]
+#[cfg(unix)]
+#[cfg_attr(docsrs, doc(cfg(unix)))]
+// rustdoc-stripper-ignore-next
 /// Create a `Future` that will resolve once the given UNIX signal is raised
 ///
 /// The `Future` must be spawned on an `Executor` backed by a `glib::MainContext`.
-pub fn unix_signal_future(signum: i32) -> Box<dyn Future<Output = ()> + std::marker::Unpin + Send> {
-    unix_signal_future_with_priority(::PRIORITY_DEFAULT, signum)
+pub fn unix_signal_future(signum: i32) -> Pin<Box<dyn Future<Output = ()> + Send + 'static>> {
+    unix_signal_future_with_priority(crate::Priority::default(), signum)
 }
 
-#[cfg(any(unix, feature = "dox"))]
+#[cfg(unix)]
+#[cfg_attr(docsrs, doc(cfg(unix)))]
+// rustdoc-stripper-ignore-next
 /// Create a `Future` that will resolve once the given UNIX signal is raised
 ///
 /// The `Future` must be spawned on an `Executor` backed by a `glib::MainContext`.
 pub fn unix_signal_future_with_priority(
     priority: Priority,
     signum: i32,
-) -> Box<dyn Future<Output = ()> + std::marker::Unpin + Send> {
-    Box::new(SourceFuture::new(move |send| {
+) -> Pin<Box<dyn Future<Output = ()> + Send + 'static>> {
+    Box::pin(SourceFuture::new(move |send| {
         let mut send = Some(send);
-        ::unix_signal_source_new(signum, None, priority, move || {
+        crate::unix_signal_source_new(signum, None, priority, move || {
             let _ = send.take().unwrap().send(());
-            Continue(false)
+            ControlFlow::Break
         })
     }))
 }
 
+// rustdoc-stripper-ignore-next
 /// Represents a `Stream` around a `glib::Source`. The stream will
 /// be provide all values that are provided by the source
 pub struct SourceStream<F, T> {
@@ -213,8 +235,9 @@ impl<F, T> Unpin for SourceStream<F, T> {}
 
 impl<F, T: 'static> SourceStream<F, T>
 where
-    F: FnOnce(mpsc::UnboundedSender<T>) -> Source + Send + 'static,
+    F: FnOnce(mpsc::UnboundedSender<T>) -> Source + 'static,
 {
+    // rustdoc-stripper-ignore-next
     /// Create a new `SourceStream`
     ///
     /// The provided closure should return a newly created `glib::Source` when called
@@ -230,7 +253,7 @@ where
 
 impl<F, T> Stream for SourceStream<F, T>
 where
-    F: FnOnce(mpsc::UnboundedSender<T>) -> Source + Send + 'static,
+    F: FnOnce(mpsc::UnboundedSender<T>) -> Source + 'static,
 {
     type Item = T;
 
@@ -265,7 +288,7 @@ where
         // At this point we must have a receiver
         let res = {
             let &mut (_, ref mut receiver) = source.as_mut().unwrap();
-            receiver.poll_next_unpin(ctx)
+            Pin::new(receiver).poll_next(ctx)
         };
         #[allow(clippy::match_wild_err_arm)]
         match res {
@@ -281,6 +304,19 @@ where
     }
 }
 
+impl<F, T> FusedStream for SourceStream<F, T>
+where
+    F: FnOnce(mpsc::UnboundedSender<T>) -> Source + 'static,
+{
+    fn is_terminated(&self) -> bool {
+        self.create_source.is_none()
+            && self
+                .source
+                .as_ref()
+                .map_or(true, |(_, receiver)| receiver.is_terminated())
+    }
+}
+
 impl<T, F> Drop for SourceStream<T, F> {
     fn drop(&mut self) {
         // Get rid of the source, we don't care anymore if it still triggers
@@ -290,80 +326,86 @@ impl<T, F> Drop for SourceStream<T, F> {
     }
 }
 
+// rustdoc-stripper-ignore-next
 /// Create a `Stream` that will provide a value every given number of milliseconds.
 ///
 /// The `Future` must be spawned on an `Executor` backed by a `glib::MainContext`.
-pub fn interval_stream(value: u32) -> Box<dyn Stream<Item = ()> + std::marker::Unpin + Send> {
-    interval_stream_with_priority(::PRIORITY_DEFAULT, value)
+pub fn interval_stream(value: Duration) -> Pin<Box<dyn Stream<Item = ()> + Send + 'static>> {
+    interval_stream_with_priority(crate::Priority::default(), value)
 }
 
+// rustdoc-stripper-ignore-next
 /// Create a `Stream` that will provide a value every given number of milliseconds.
 ///
 /// The `Future` must be spawned on an `Executor` backed by a `glib::MainContext`.
 pub fn interval_stream_with_priority(
     priority: Priority,
-    value: u32,
-) -> Box<dyn Stream<Item = ()> + std::marker::Unpin + Send> {
-    Box::new(SourceStream::new(move |send| {
-        ::timeout_source_new(value, None, priority, move || {
+    value: Duration,
+) -> Pin<Box<dyn Stream<Item = ()> + Send + 'static>> {
+    Box::pin(SourceStream::new(move |send| {
+        crate::timeout_source_new(value, None, priority, move || {
             if send.unbounded_send(()).is_err() {
-                Continue(false)
+                ControlFlow::Break
             } else {
-                Continue(true)
+                ControlFlow::Continue
             }
         })
     }))
 }
 
+// rustdoc-stripper-ignore-next
 /// Create a `Stream` that will provide a value every given number of seconds.
 ///
 /// The `Stream` must be spawned on an `Executor` backed by a `glib::MainContext`.
-pub fn interval_stream_seconds(
-    value: u32,
-) -> Box<dyn Stream<Item = ()> + std::marker::Unpin + Send> {
-    interval_stream_seconds_with_priority(::PRIORITY_DEFAULT, value)
+pub fn interval_stream_seconds(value: u32) -> Pin<Box<dyn Stream<Item = ()> + Send + 'static>> {
+    interval_stream_seconds_with_priority(crate::Priority::default(), value)
 }
 
+// rustdoc-stripper-ignore-next
 /// Create a `Stream` that will provide a value every given number of seconds.
 ///
 /// The `Stream` must be spawned on an `Executor` backed by a `glib::MainContext`.
 pub fn interval_stream_seconds_with_priority(
     priority: Priority,
     value: u32,
-) -> Box<dyn Stream<Item = ()> + std::marker::Unpin + Send> {
-    Box::new(SourceStream::new(move |send| {
-        ::timeout_source_new_seconds(value, None, priority, move || {
+) -> Pin<Box<dyn Stream<Item = ()> + Send + 'static>> {
+    Box::pin(SourceStream::new(move |send| {
+        crate::timeout_source_new_seconds(value, None, priority, move || {
             if send.unbounded_send(()).is_err() {
-                Continue(false)
+                ControlFlow::Break
             } else {
-                Continue(true)
+                ControlFlow::Continue
             }
         })
     }))
 }
 
-#[cfg(any(unix, feature = "dox"))]
+#[cfg(unix)]
+#[cfg_attr(docsrs, doc(cfg(unix)))]
+// rustdoc-stripper-ignore-next
 /// Create a `Stream` that will provide a value whenever the given UNIX signal is raised
 ///
 /// The `Stream` must be spawned on an `Executor` backed by a `glib::MainContext`.
-pub fn unix_signal_stream(signum: i32) -> Box<dyn Stream<Item = ()> + std::marker::Unpin + Send> {
-    unix_signal_stream_with_priority(::PRIORITY_DEFAULT, signum)
+pub fn unix_signal_stream(signum: i32) -> Pin<Box<dyn Stream<Item = ()> + Send + 'static>> {
+    unix_signal_stream_with_priority(crate::Priority::default(), signum)
 }
 
-#[cfg(any(unix, feature = "dox"))]
+#[cfg(unix)]
+#[cfg_attr(docsrs, doc(cfg(unix)))]
+// rustdoc-stripper-ignore-next
 /// Create a `Stream` that will provide a value whenever the given UNIX signal is raised
 ///
 /// The `Stream` must be spawned on an `Executor` backed by a `glib::MainContext`.
 pub fn unix_signal_stream_with_priority(
     priority: Priority,
     signum: i32,
-) -> Box<dyn Stream<Item = ()> + std::marker::Unpin + Send> {
-    Box::new(SourceStream::new(move |send| {
-        ::unix_signal_source_new(signum, None, priority, move || {
+) -> Pin<Box<dyn Stream<Item = ()> + Send + 'static>> {
+    Box::pin(SourceStream::new(move |send| {
+        crate::unix_signal_source_new(signum, None, priority, move || {
             if send.unbounded_send(()).is_err() {
-                Continue(false)
+                ControlFlow::Break
             } else {
-                Continue(true)
+                ControlFlow::Continue
             }
         })
     }))
@@ -371,27 +413,28 @@ pub fn unix_signal_stream_with_priority(
 
 #[cfg(test)]
 mod tests {
+    use std::{thread, time::Duration};
+
+    use futures_util::{future::FutureExt, stream::StreamExt};
+
     use super::*;
-    use std::thread;
 
     #[test]
     fn test_timeout() {
         let c = MainContext::new();
 
-        let res = c.block_on(timeout_future(20));
-
-        assert_eq!(res, ());
+        c.block_on(timeout_future(Duration::from_millis(20)));
     }
 
     #[test]
     fn test_timeout_send() {
         let c = MainContext::new();
-        let l = ::MainLoop::new(Some(&c), false);
+        let l = crate::MainLoop::new(Some(&c), false);
 
         let l_clone = l.clone();
-        c.spawn(timeout_future(20).then(move |()| {
+        c.spawn(timeout_future(Duration::from_millis(20)).then(move |()| {
             l_clone.quit();
-            future::ready(())
+            futures_util::future::ready(())
         }));
 
         l.run();
@@ -405,18 +448,16 @@ mod tests {
 
         {
             let count = &mut count;
-            let res = c.block_on(
-                interval_stream(20)
+            c.block_on(
+                interval_stream(Duration::from_millis(20))
                     .take(2)
                     .for_each(|()| {
-                        *count = *count + 1;
+                        *count += 1;
 
-                        future::ready(())
+                        futures_util::future::ready(())
                     })
                     .map(|_| ()),
             );
-
-            assert_eq!(res, ());
         }
 
         assert_eq!(count, 2);
@@ -426,14 +467,14 @@ mod tests {
     fn test_timeout_and_channel() {
         let c = MainContext::default();
 
-        let res = c.block_on(timeout_future(20).then(|()| {
+        let res = c.block_on(timeout_future(Duration::from_millis(20)).then(|()| {
             let (sender, receiver) = oneshot::channel();
 
             thread::spawn(move || {
                 sender.send(1).unwrap();
             });
 
-            receiver.then(|i| future::ready(i.unwrap()))
+            receiver.then(|i| futures_util::future::ready(i.unwrap()))
         }));
 
         assert_eq!(res, 1);
